#include <unistd.h>
#include <fcntl.h>
#include "globdef.h"
#include "uidef.h"
#include "sdrdef.h"
#include "conf.h"
#include "fft1def.h"
#include "screendef.h"
#include "rusage.h"
#include "thrdef.h"
#include "options.h"

#define MAX_LIBUSB_BULK_BUFFER 16320


#if PERSEUSLIB_PRESENT == 1
#include <perseus-sdr.h>
perseus_descr *descr;
#endif

#define SDR14_DEVICE_DRIVER "/dev/linrad_ft245"
// Perseus receiver USB buffering scheme definitions (never touch)
#define PERSEUS_SAMPLESPERFRAME	170   // Samples in one FX2 USB transaction
#define PERSEUS_COMPONENTSPERSAMPLE 2 // I & Q components
#define PERSEUS_BYTESPERCOMPONENT   3 // 24 bit data
// Derived defines
#define PERSEUS_BYTESPERSAMPLE (PERSEUS_BYTESPERCOMPONENT*PERSEUS_COMPONENTSPERSAMPLE)
#define PERSEUS_BYTESPERFRAME (PERSEUS_SAMPLESPERFRAME*PERSEUS_BYTESPERSAMPLE)

extern P_PERSEUS pers;
extern float perseus_filter_cutoff[MAX_PERSEUS_FILTER+1];
extern char *perseus_bitstream_names[MAX_PERSEUS_RATES];
int perseus_bytesperbuf;

void start_excalibur_read(void){}
void lir_excalibur_read(void){}
void set_excalibur_frequency(void){}
void excalibur_stop(void){}


int load_excalibur_library(char* buf)
{
(void)buf;
return 0;
}


void open_excalibur(void)
{
}

void close_excalibur(void)
{
}

void select_excalibur_mode(int *line)
{
(void)line;
}

void set_excalibur_att(void)
{
/*
int i;
i=-fg.gain;
//ig31_device->lpVtbl->SetAttenuator(ig31_device, i);
*/
}


void set_perseus_frequency(void)
{
#if PERSEUSLIB_PRESENT == 1
double dt1;
dt1=fg.passband_center*1000000+0.5;
if(dt1 < 0 || dt1 > adjusted_sdr_clock/2)
  {
  fg.passband_center=7.05;
  dt1=fg.passband_center*1000000;
  }
perseus_set_ddc_center_freq(descr, dt1, pers.presel);
#endif
}

void fill_perseus_rates(void)
{
#if PERSEUSLIB_PRESENT == 1
if (perseus_get_sampling_rates (descr, perseus_rates, PERSEUS_RATES_BUFSIZE))
  {
  lirerr(1196);
  return;
  }
no_of_perseus_rates = 0;
while (perseus_rates[no_of_perseus_rates] > 1) 
  {
  no_of_perseus_rates++;
  }
#endif
}

void perseus_stop(void)
{
#if PERSEUSLIB_PRESENT == 1
perseus_stop_async_input(descr);
lir_sleep(10000);
#endif
}



int perseus_callback(void *buf, int buf_size, void *extra)
{
int i;
register char c1;
char *pBuf;
pBuf=extra;
pBuf=(void*)buf;
c1=0;
// This callback gets called whenever a buffer is ready
// from the USB interface
for(i=0; i<buf_size; i+=6)
  {
  timf1_char[timf1p_sdr+3]=pBuf[i+2];
  c1=pBuf[i+2]&0x80;
  timf1_char[timf1p_sdr+2]=pBuf[i+1];
  timf1_char[timf1p_sdr+1]=pBuf[i  ];
  c1|=pBuf[i];
  timf1_char[timf1p_sdr  ]=0;
  timf1_char[timf1p_sdr+7]=pBuf[i+5];
  timf1_char[timf1p_sdr+6]=pBuf[i+4];
  timf1_char[timf1p_sdr+5]=pBuf[i+3];
  timf1_char[timf1p_sdr+4]=0;
  timf1p_sdr=(timf1p_sdr+8)&timf1_bytemask;
  }
if( (c1&1) != 0)
  {
  ad_maxamp[0]=0x7fff;
  ad_maxamp[1]=0x7fff;
  }
if(((timf1p_sdr-timf1p_pa+timf1_bytes)&timf1_bytemask) >=
              (int)snd[RXAD].block_bytes)lir_set_event(EVENT_PERSEUS_INPUT);
return 0;
}

void start_perseus_read(void)
{
#if PERSEUSLIB_PRESENT == 1
int i;
sdr=-1;
perseus_bytesperbuf=snd[RXAD].block_bytes/PERSEUS_BYTESPERFRAME;
perseus_bytesperbuf*=PERSEUS_BYTESPERFRAME;
if(perseus_bytesperbuf > MAX_LIBUSB_BULK_BUFFER)
  {
  perseus_bytesperbuf=MAX_LIBUSB_BULK_BUFFER/PERSEUS_BYTESPERFRAME;
  perseus_bytesperbuf*=PERSEUS_BYTESPERFRAME;
  }
timf1p_sdr=timf1p_pa;
lir_sched_yield();
i=perseus_start_async_input(descr,perseus_bytesperbuf, perseus_callback, NULL);
if(i < 0) 
  {
  lirerr(1195);
  return;
  }
sdr=0;
#endif
}

void lir_perseus_read(void)
{
while(((timf1p_sdr-timf1p_pa+timf1_bytes)&timf1_bytemask) < (int)snd[RXAD].block_bytes)
  {
  lir_await_event(EVENT_PERSEUS_INPUT);
  }
}

int perseus_load_proprietary_code(int rate_no)
{
#if PERSEUSLIB_PRESENT == 1
int i;
i=perseus_set_sampling_rate_n(descr, rate_no);
if(i<0) return 1194;
return 0;
#else
return rate_no;
#endif
}

int perseus_eeprom_read(unsigned char *buf, int addr, unsigned char count)
{
#if PERSEUSLIB_PRESENT == 1
eeprom_prodid *prodid;
int i;
char s[80];
prodid=(void*) buf;
(void) buf;
(void) addr;
(void) count;
i=perseus_get_product_id(descr,prodid);
if(i<0)
  {
  sprintf(s,"get product id error: %s", perseus_errorstr());
  clear_screen();
  lir_text(5,5,s);
  lir_text(5,7,press_any_key);
  await_processed_keyboard();
  lirerr(1193);
  return 1;
  }
return 0;
#else
buf[0]=addr;
return count;
#endif
}

void open_perseus(void)
{
#if PERSEUSLIB_PRESENT == 1
int i;
#if PERS_DEBUG == 1
perseus_set_debug(3);
#endif
if(sdr != -1)
  {
#if PERS_DEBUG == 1
  fprintf(stderr,"lsdr.c: error sdr=-1\n");
#endif
  return;
  }
if(perseus_init() == 0)
  {
#if PERS_DEBUG == 1
  fprintf(stderr,"lsdr.c: error perseus_init() failed\n");
#endif
  lir_text(3,3,"Could not init. Is Perseus connected with power on?");
conerr:;
  return;
  }
descr=perseus_open(0);
if(descr == NULL)
  {
#if PERS_DEBUG == 1
  fprintf(stderr,"lsdr.c: error perseus_open() failed\n");
#endif
  clear_screen();
  lir_text(3,2,"Could not open Perseus. Do you have write permission to USB?");
  lir_text(3,3,"Maybe you should use sudo (or run as root)");
  goto conerr;
  }
i=perseus_firmware_download(descr,NULL);
if(i < 0)
  {
#if PERS_DEBUG == 1
  fprintf(stderr,"lsdr.c: error perseus_firmware_download(...) failed\n");
#endif
  close_perseus();
  return;
  }
sdr=0;  
#endif
}

void perseus_store_att(unsigned char idAtt)
{
#if PERSEUSLIB_PRESENT == 1
perseus_set_attenuator(descr, idAtt);
#else
(void)idAtt;
#endif
}

void perseus_store_presel(unsigned char idPresel)
{
(void) idPresel;
}

void close_perseus(void)
{
#if PERSEUSLIB_PRESENT == 1
perseus_close(descr);
perseus_exit();
sdr=-1;
#endif
}

int zrwx;
void lir_sdr14_write(void *s, int bytes)
{
zrwx=write(sdr,s,bytes);
}

int lir_sdr14_read(void *s,int bytes)
{
int i;
i=read(sdr, s, bytes);
return i;
}

void open_sdr14(void)
{
SNDLOG"Trying to open %s\n",SDR14_DEVICE_DRIVER);
sdr=open( SDR14_DEVICE_DRIVER ,O_RDWR );
}

void close_sdr14(void)
{
int i;
i=fcntl(sdr, F_GETFD);
i|=O_NONBLOCK;
fcntl(sdr,F_SETFD,i);
lir_sleep(10000);
close(sdr);
sdr=-1;
}

